import java.awt.*;
import java.awt.image.*;
import java.net.*;
import java.io.*;

/**************************************************************/
/* Tokenklasse zur Erzeugung und Bewegung des frei kreisenden */
/* Tokens und der Datenrahmen                                 */
/**************************************************************/
class tokenelement
{
   private static final int pointmax = 100;
   private static final int pu = 360;
   private static final int dist = 4;

   private netpaint box;
   private Image image;
   private token parent;
   private static int anz = 0;
   private int i = 0;
   private int xcenter, ycenter, xradius, yradius, x2radius, y2radius;
   private int maxComponentWidth, maxComponentHeight;
   private int containerWidth, containerHeight;
   private double angle = 0;
   private Point[] coords = new Point[pointmax];
   private Color color[] = new Color [pointmax];
   private boolean reverse[] = new boolean [pointmax];   //umgekehrte Richtung
   private boolean emptymode[] = new boolean [pointmax]; //Ausgefuellter Kreis
   public int krpos[] = new int [pointmax];
   public int startstation = -1;
   public int stopstation = -1;

   public int maxframe = 0;

   public int t_anz = 1;
   public int t_stat = 0; // Variablen fuer Run-Methode
   public int zstart = 2;
   public int zaehler = 0;
   public int iresume = 0;


   public tokenelement(netpaint c, Image i, token p)
   {
      box = c; image = i; parent = p;
      anz++;
      krpos[0] = 0;
   }

   public boolean getreverse()
   {
      return reverse[0];
   }

   private void setxy()
   {
      Dimension d = box.size();

      maxComponentWidth = image.getWidth(box)-5;
      maxComponentHeight = image.getHeight(box);

      containerWidth = d.width;
      containerHeight = d.height;
      xcenter = containerWidth / 2;
      ycenter = containerHeight / 2;
      xradius = (containerWidth - maxComponentWidth-10) / 2;
      yradius = (containerHeight - maxComponentHeight-10) / 2;
      x2radius = (containerWidth - maxComponentWidth-40) / 2;
      y2radius = (containerHeight - maxComponentHeight-40) / 2;
   }

   private double calcangle(int i)
   {
      return (2 * Math.PI * i / pu);
   }

   public void draw(int nr, Color col, boolean reversestat)
   {
      int i;

      Graphics g = box.getGraphics();
      setxy();

      if (reversestat)
         i = krpos[0] + (dist * nr);
      else
         i = krpos[0] - (dist * nr);
      while (i < 0) i += 360;
      while (i >= 360) i -= 360;
      krpos[nr] = i;
      angle = calcangle(krpos[nr]);
      if (reversestat)
      {
         coords[nr] = new Point(xcenter + (int)(Math.cos(angle) * xradius),
                              ycenter + (int)(Math.sin(angle) * yradius));
         reverse[nr] = true;
      } else
      {
         coords[nr] = new Point(xcenter + (int)(Math.cos(angle) * x2radius),
                              ycenter + (int)(Math.sin(angle) * y2radius));
         reverse[nr] = false;
      }
      if (col != null) color[nr] = col; else color[nr] = color[0];
      emptymode[nr] = false;
      g.setColor(col);
      g.setXORMode(box.getBackground());
      g.fillOval(coords[nr].x-5,coords[nr].y-5,10,10);
      System.out.flush();
      g.dispose();
   }

   public void erase(int nr, boolean emptymode)
   {
      Graphics g = box.getGraphics();
      setxy();

      g.setColor(color[nr]);
      g.setXORMode(box.getBackground());
      if (emptymode)
         g.drawOval(coords[nr].x-5,coords[nr].y-5,10,10);
      else
         g.fillOval(coords[nr].x-5,coords[nr].y-5,10,10);
      if (nr != 0) coords[nr] = null;
      System.out.flush();
      g.dispose();
   }

   /* Rückgabewerte:

      1: normal
      2: Sendevorgang
      3: Empfangsvorgang
      4: Loeschvorgang
      5: Pruefbitfehler

   */
   public int move(int nr, int startnr)
   {
      int t_status = 1;
      int i;

      Graphics g = box.getGraphics();

      if (reverse[nr])
         krpos[nr]--;
      else
         krpos[nr]++;
      while (krpos[nr] < 0) krpos[nr] += 360;
      while (krpos[nr] >= 360) krpos[nr] -= 360;
      i = krpos[nr];
      angle = calcangle(krpos[nr]);
      g.setColor(color[nr]);
      g.setXORMode(box.getBackground());
      if (emptymode[nr])
         g.drawOval(coords[nr].x-5,coords[nr].y-5,10,10);
      else
         g.fillOval(coords[nr].x-5,coords[nr].y-5,10,10);
      if (reverse[nr])
      {
         coords[nr].x = xcenter + (int)(Math.cos(angle) * x2radius);
         coords[nr].y = ycenter + (int)(Math.sin(angle) * y2radius);
      } else
      {
         coords[nr].x = xcenter + (int)(Math.cos(angle) * xradius);
         coords[nr].y = ycenter + (int)(Math.sin(angle) * yradius);
      }
      if ((i % Math.round(360.0/(float)box.getAnz())) == 0)
      {
         int stnr = (i / (360 / box.getAnz()));
         if (stnr < box.getAnz())
         {
            //Abfrage fuer Animation
            //Sendevorgang abgeschlossen code * 4 *
            if ((startstation != -1) && (startstation == stnr) &&
               (color[nr] != Color.red) &&
               (emptymode[nr]))
            {
               color[nr] = Color.red;
               emptymode[nr] = false;
               t_status = 4;
               parent.sended[stnr] = false;
               if (nr == 0) box.sta.status.appendText("Sending process ended.\n");
            } else
            //Empfangsmodus code * 3 *
            if ((stnr == stopstation) &&
               (color[nr] != Color.red) &&
               (!emptymode[nr]) &&
               ((box.st[stnr].getfunction() & 1) == 1))
            {
               emptymode[nr] = true;
               t_status = 3;
               stopstation = stnr;
               if (nr == 0) box.sta.status.appendText("Data frames receiving ...\n");
            } else
            //Sendevorgang code * 2 *
            if ((box.st[stnr].getstatus() >= 100) &&
               (box.st[stnr].getstatus() < 200) &&
               (color[nr] == Color.red) &&
               (!parent.sended[stnr]))
            {
               color[nr] = box.farben[box.st[stnr].getdist()];
               t_status = 2;
               startstation = stnr;
               parent.sended[stnr] = true;
               stopstation = box.st[stnr].getstatus() - 100;
               box.sta.status.appendText("Data frames sending ...\n");
            }
            if ((box.st[stnr].getfunction() & 4) == 4)
            {
               if ((stnr != startstation) || (nr == 0))
               {
                  int tmpst = -1;

                  if (!reverse[nr]) {
                     if (stnr == 0) tmpst = box.getAnz()-1; else tmpst = stnr - 1;
                  } else {
                     if (stnr == box.getAnz()-1) tmpst = 0; else tmpst = stnr +1;
                  }
                  if (!((box.st[tmpst].getfunction() & 4) == 4)) reverse[nr] = !(reverse[nr]);
               }
            }
         }
      }
      g.setColor(color[nr]);
      g.setXORMode(box.getBackground());
      if (emptymode[nr])
         g.drawOval(coords[nr].x-5,coords[nr].y-5,10,10);
      else
         g.fillOval(coords[nr].x-5,coords[nr].y-5,10,10);
      System.out.flush();
      g.dispose();
      return t_status;
   }
}

class token implements Runnable
{
   private Thread th = null;

   private static final int dist = 4;
   private static final int tokenmax = 20;

   private netpaint box;
   public int sleepTime = 50;
   private Image image;
   private boolean endless = true;
   private boolean[] multiply = new boolean[tokenmax];
   tokenelement[] t = new tokenelement[tokenmax];
   private boolean[] hide = new boolean[tokenmax];

   private boolean[] tokenactive = new boolean[tokenmax];
   private int[] tstart = new int[tokenmax];

   public boolean[] sended = new boolean[15]; //Sendevorgang an der Station ausgefuehrt

   public token(netpaint c, Image i)
   {
      box = c; image = i;
   }

   public boolean animstat()
   {
      boolean help = false;

      for (int i = 0; i < tokenmax; i++)
         if (multiply[i] == true) help = true;
      return help;
   }

   public boolean isAlive()
   {
      if (th == null) return false;
      return th.isAlive();
   }

   public void suspend()
   {
      if (th != null) th.suspend();
   }

   public void resume()
   {
      if (th != null) th.resume();
   }

   public void start()
   {
      if (th == null)
      {
	      th = new Thread(box.sta.gr, this);
         th.start();
	   }
   }

   public void stop()
   {
      if ((th != null) && (th.isAlive()))
	      th.stop();
      th = null;
   }

   public void run()
   {
      int i = 0;
      int z;

      for (i = 0; i < tokenmax; i++)
      {
         t[i] = new tokenelement(box, image, this);
         tokenactive[i] = false;
         multiply[i] = false;
         hide[i] = false;
      }
      for (i = 0; i < 15; i++) sended[i] = false;
      tokenactive[0] = true;
      t[0].draw(0, Color.red, false);
      do
      {
         for (i = 0; i < tokenmax; i++)
         {
            if (tokenactive[i])
            {
               if (!hide[i])
               {
                  t[i].t_stat = t[i].move(0,0);
                  if (t[i].t_stat == 2) //Sendevorgang
                  {
                     multiply[i] = true;
                     tstart[i] = t[i].krpos[0];
                  }
                  if (t[i].t_stat == 4) //Loeschvorgang
                  {
                     t[i].erase(0, false);
                     hide[i] = true;
                  }
               }
               if (multiply[i])
               {
                  if (t[i].t_anz < 5) t[i].zaehler++;
                  //Neue Token auf den Ring setzen
                  if (((t[i].zaehler % dist) == 0) &&
                  (t[i].t_anz < 5))
                  {
                     t[i].draw(t[i].t_anz, null, t[i].getreverse());
                     t[i].t_anz++;
                  }
                  if ((t[i].t_anz == 5) && (t[i].zaehler != 0))
                  {
                     int itmp;

                     t[i].zaehler++;  //Zaehler weiter zaehlen fuer neues Token
                     if ((t[i].zaehler % (dist+3)) == 0)
                     {
                        itmp = i+1;
                        if (itmp >= tokenmax)
                        {
                          if (!tokenactive[0])
                          {
                            itmp = 0;
                            t[itmp].krpos[0] = tstart[i];
                            t[itmp].draw(0, Color.red, t[tokenmax-1].getreverse());
                            tokenactive[itmp] = true;
                          }
                          t[i].zaehler = 0;
                        } else
                        {
                          t[itmp].krpos[0] = tstart[i];
                          t[itmp].draw(0, Color.red, t[itmp-1].getreverse());
                          tokenactive[itmp] = true;
                          t[i].zaehler = 0;
                        }
                     }
                  }
                  for (z = t[i].zstart; z <= t[i].t_anz; z++)
                  {
                     if (hide[i])
                        t[i].t_stat = t[i].move(z-1,t[i].zstart-1);
                     else
                        t[i].t_stat = t[i].move(z-1,0);
                     if (t[i].t_stat == 4)
                     {
                        t[i].erase(z-1, false);
                        t[i].zstart++;
                     }
                     if (t[i].zstart > t[i].t_anz)
                     {
                        multiply[i] = false;
                        box.sendoff(t[i].startstation, t[i].stopstation);
                        t[i].zaehler = 0;
                        t[i].zstart = 2;
                        t[i].t_anz = 1;
                        hide[i] = false;
               			tokenactive[i] = false;
                     }
                  }
               } //multiply
            } //tokenactive
            System.out.flush();
         }
         try {th.sleep(sleepTime);}
         catch(InterruptedException e) {};
      } while (endless);
   }

   public void setsleep(int sl)
   {
      sleepTime = sl;
   }
}

/**************************************************************/
/* Klasse zum Zeichnen des Netzwerkes und Steuerung der       */
/* Animation                                                  */
/**************************************************************/
class netpaint extends Canvas
{
   private static final int MAXSTATIONS = 15;
   public static final Color [] farben = {Color.green,Color.blue,Color.cyan,
                                          Color.magenta,Color.orange,
                                          Color.pink,Color.black};
   private static boolean breakstat = false;

   private static int anz = 0;
   private static boolean handcursor = false;
   private Image image, image2;
   station[] st;
   private Point[] squares = new Point[MAXSTATIONS];
   private int farbnr = 0;                // Aktuelle Markierungsfarbe
   fddi sta;
   private int stangle;
   private int stopangle;
   private int selectmode = 1;

   int maxComponentWidth = 0;
   int maxComponentHeight = 0;
   int xcenter, ycenter;
   int xradius, yradius;
   int x2radius, y2radius;

   public boolean paintactive = false;

   token t;

   private boolean sendmode = false;

   public netpaint(Image i, Image i2, fddi s)
   {
      image = i;
      image2 = i2;
      sta = s;
      st = new station[MAXSTATIONS];
      st[0]= new station();
      st[0].setfunction(1); //1: aktiv
      st[1]= new station();
      st[1].setfunction(1);
      anz = 2;

      //temporaer
      stangle = -1;
      stopangle = -1;
   }

   public int getAnz()
   {
      return anz;
   }

   public void setselectmode(int mode)
   {
      selectmode = mode;
   }

   public void setsend(boolean sm)
   {
      sendmode = sm;
   }

   public void sendoff(int sendnr, int recnr)
   {
      int f;
      int dist = 0;

      Graphics g = this.getGraphics();
      g.setColor(getBackground());
      st[sendnr].setstatus(1);
      if (st[recnr].getstatus() < 100) st[recnr].setstatus(1);
      dist = st[sendnr].getdist();
      g.drawRect(squares[sendnr].x-6-dist, squares[sendnr].y-4-dist,
         maxComponentWidth+8+(2*dist), maxComponentHeight+8+(2*dist));
      g.drawOval(squares[recnr].x-6-dist, squares[recnr].y-4-dist,
         maxComponentWidth+8+(2*dist), maxComponentHeight+8+(2*dist));
      g.dispose();
   }

   public boolean token_lives()
   {
      if (( t != null) && (t.isAlive())) return true;
      return false;
   }

   public boolean anim_lives()
   {
      if ((t != null) && (t.animstat())) return true;
      return false;
   }

   private int find(int x, int y)
   {
      for (int z = 0; z < anz; z++)
      {
         if (squares[z] != null)
         {
            if (squares[z].x <= x && x <= squares[z].x+maxComponentWidth &&
               squares[z].y <= y && y <= squares[z].y+maxComponentHeight)
               return z;
         }
      }
      return -1;
   }

   private int findstation(int x, int y)
   {
      int i;
      int xc, yc; //rel. Koordinaten zum Ursprung
      int deg;

      Graphics g = this.getGraphics();
      Dimension d = size();

      int containerWidth = d.width;
      int containerHeight = d.height;

      maxComponentWidth = image.getWidth(this)-5;
      maxComponentHeight = image.getHeight(this);
      xradius = (containerWidth - maxComponentWidth-10) / 2;
      yradius = (containerHeight - maxComponentHeight-10) / 2;
      xcenter = containerWidth / 2;
      ycenter = containerHeight / 2;

      xc = x - xcenter;
      yc = (containerHeight - y) - ycenter;

      //Cosinusfunktion
      deg = (int)Math.round(Math.acos(xc/Math.sqrt((xc*xc)+(yc*yc))) * 180 / Math.PI);
      if (yc < 0)
      {
         deg = 360 - deg;
      }

      i = anz-(int)Math.round(deg/(360/anz))-1;

      return i;
   }

   public boolean mouseMove(Event evt, int x, int y)
   {
      if (find(x, y) >= 0)
      {
         if ((!handcursor) && (sendmode))
         {
            sta.setCursor(Frame.CROSSHAIR_CURSOR);
            handcursor = true;
         }
      }
      else
      {
         if (handcursor)
         {
            sta.setCursor(Frame.DEFAULT_CURSOR);
            handcursor = false;
         }
      }
      return true;
   }

   public boolean mouseDown(Event evt, int x, int y)
   {
      int f;

      f = find(x, y);
      Graphics g = this.getGraphics();

      maxComponentWidth = image.getWidth(this)-5;
      maxComponentHeight = image.getHeight(this);

      if ((evt.clickCount == 1) && (f >= 0) && (sendmode))
      {
         if (selectmode == 1)  //Sende-/Empfangsstation auswaehlen
         {
            if ((st[f].getfunction() & 1) == 0)
            {
               sta.status.appendText("Station is off-line!\n");
            } else
            {
               if ((stangle == -1) && (st[f].getstatus() < 100))
               {
                  stangle = f;
                  sta.status.appendText("Sending station No. "+f+" selected!\n");
                  g.setColor(farben[farbnr]);
                  g.setXORMode(getBackground());
                  g.drawRect(squares[f].x-6-farbnr, squares[f].y-4-farbnr,
                           maxComponentWidth+8+(2*farbnr), maxComponentHeight+8+(2*farbnr));
               }
               else
               {
                  if ((stangle != -1) && (f != stangle))
                  {
                     stopangle = f;
                     sta.status.appendText("Receiving station No. "+f+" selected!\n");
                     st[stangle].setstatus(100+stopangle); //Zieladresse in Startstation schreiben
                     st[stangle].setdist(farbnr);
                     if (st[stopangle].getstatus() < 100) st[stopangle].setstatus(2);
                     g.setColor(farben[farbnr]);
                     g.setXORMode(getBackground());
                     g.drawOval(squares[f].x-6-farbnr, squares[f].y-4-farbnr,
                           maxComponentWidth+8+(2*farbnr), maxComponentHeight+8+(2*farbnr));
                     stangle = -1; stopangle = -1;
                     farbnr++;
                     if (farbnr > 6) farbnr = 0;
                  }
               }
            }
         } else if (selectmode == 2)  //Station ein-/ausschalten
         {
            if (((st[f].getfunction() & 1) == 1) && (st[f].getstatus() == 1))
            {
               st[f].setfunction(st[f].getfunction() & 6);
               sta.status.appendText("Station No. "+f+" switched off.\n");
               g.setColor(Color.red);
               g.setXORMode(getBackground());
               g.drawLine(squares[f].x, squares[f].y,
                          maxComponentWidth+squares[f].x, maxComponentHeight+squares[f].y);
               g.drawLine(squares[f].x+1, squares[f].y,
                          maxComponentWidth+squares[f].x+1, maxComponentHeight+squares[f].y);
               g.drawLine(maxComponentWidth+squares[f].x, squares[f].y,
                          squares[f].x, maxComponentHeight+squares[f].y);
               g.drawLine(maxComponentWidth+squares[f].x-1, squares[f].y,
                          squares[f].x-1, maxComponentHeight+squares[f].y);
            }
            else if ((st[f].getfunction() & 1) == 0)
            {
               st[f].setfunction(st[f].getfunction() | 1);
               sta.status.appendText("Station No. "+f+" is online.\n");
               g.setColor(Color.red);
               g.setXORMode(getBackground());
               g.drawLine(squares[f].x, squares[f].y,
                          maxComponentWidth+squares[f].x, maxComponentHeight+squares[f].y);
               g.drawLine(squares[f].x+1, squares[f].y,
                          maxComponentWidth+squares[f].x+1, maxComponentHeight+squares[f].y);
               g.drawLine(maxComponentWidth+squares[f].x, squares[f].y,
                          squares[f].x, maxComponentHeight+squares[f].y);
               g.drawLine(maxComponentWidth+squares[f].x-1, squares[f].y,
                          squares[f].x-1, maxComponentHeight+squares[f].y);
            } else
               sta.status.appendText("Station No. "+f+" cannot be switched off.\n");
         }
      }
      if ((evt.clickCount == 1) && (selectmode == 3))
      {
         if (anim_lives())
         {
            sta.status.appendText("No selection possible during animation!\n");
         } else
         {
            int fs, ss;
            fs = findstation(x, y);
            ss = fs+1; if (ss >= anz) ss = 0;
            if (breakstat)
            {
               if (((st[fs].getfunction() & 4) == 4) &&
                  ((st[ss].getfunction() & 4) == 4))
               {
                  sta.status.appendText("Break repaired!\n");
                  st[fs].setfunction(st[fs].getfunction() & 3);
                  st[ss].setfunction(st[ss].getfunction() & 3);
                  breakstat = false;
               } else return true;
            } else
            {
               st[fs].setfunction(st[fs].getfunction() | 4);
               st[ss].setfunction(st[ss].getfunction() | 4);
               sta.status.appendText("Break between station "+fs+" and "+ss+"\n");
               breakstat = true;
            }
            x = xcenter+5 + (int)(Math.cos(2*Math.PI*(fs+0.5)/anz) * xradius);
            y = ycenter + (int)(Math.sin(2*Math.PI*(fs+0.5)/anz) * yradius)-5;
            g.setXORMode(getBackground());
            g.drawImage(image2,x-(maxComponentWidth/2), y-(maxComponentHeight/2), this);
         }
      }
      g.dispose();
      return true;
   }

   public boolean mouseExit(Event evt, int x, int y)
   {
      sta.setCursor(Frame.DEFAULT_CURSOR);
      return true;
   }

   public void startanim(int slt)
   {
      t = new token(this, image);
      t.setsleep(slt);
      t.start();
      sendmode = true;
   }

   public void stopanim()
   {
		if (token_lives())
      {
         t.stop();
         t = null;
	      for (int i = 0; i < anz; i++)
         {
	         st[i].setstatus(1);
            st[i].setfunction(1);
         }
         stangle = -1;
         stopangle = -1;
         repaint();
	      sendmode = false;
         breakstat = false;
      }
   }

   public void paint(Graphics g)
   {
      boolean bpp = false; //Flag fuer Breakpunkt

      paintactive = true;

      Dimension d = size();

      maxComponentWidth = image.getWidth(this)-5;
      maxComponentHeight = image.getHeight(this);

      int containerWidth = d.width;
      int containerHeight = d.height;

      xradius = (containerWidth - maxComponentWidth-10) / 2;
      yradius = (containerHeight - maxComponentHeight-10) / 2;
      x2radius = (containerWidth - maxComponentWidth-40) / 2;
      y2radius = (containerHeight - maxComponentHeight-40) / 2;
      xcenter = containerWidth / 2;
      ycenter = containerHeight / 2;

      //Kreis zeichnen
      Polygon Netzlinie = new Polygon();
      int m = 360;
      for (int i = 0; i < m; i++)
      {
         double angle = 2 * Math.PI * i / m;
         int x = xcenter + (int)(Math.cos(angle) * xradius);
         int y = ycenter + (int)(Math.sin(angle) * yradius);
         Netzlinie.addPoint(x,y);
      }
      g.drawPolygon(Netzlinie);

      Polygon Netzlinie2 = new Polygon();
      for (int i = 0; i < m; i++)
      {
         double angle = 2 * Math.PI * i / m;
         int x = xcenter + (int)(Math.cos(angle) * x2radius);
         int y = ycenter + (int)(Math.sin(angle) * y2radius);
         Netzlinie2.addPoint(x,y);
      }
      g.drawPolygon(Netzlinie2);

      //Stationen zeichnen
      for (int i = 0; i < anz; i++)
      {
         double angle = 2 * Math.PI * i / anz;
         int x = xcenter + (int)(Math.cos(angle) * (xradius-10));
         int y = ycenter + (int)(Math.sin(angle) * (yradius-10));
         g.drawImage(image, x-(maxComponentWidth/2), y-(maxComponentHeight/2), this);
         squares[i] = new Point(x-(maxComponentWidth/2), y-(maxComponentHeight/2));

         //Nummern zeichnen
         x = xcenter - 10 + (int)(Math.cos(angle) * (xradius-60));
         y = ycenter + (int)(Math.sin(angle) * (yradius-60));
         g.setColor(Color.white);
         g.drawString(""+i, x, y);

         //Breakpunkt zeichnen
         if (((st[i].getfunction() & 4) == 4) && !(bpp))
         {
            x = xcenter+5 + (int)(Math.cos(2*Math.PI*(i+0.5)/anz) * xradius);
            y = ycenter + (int)(Math.sin(2*Math.PI*(i+0.5)/anz) * yradius)-5;
            g.setXORMode(getBackground());
            g.drawImage(image2,x-(maxComponentWidth/2), y-(maxComponentHeight/2), this);
            bpp = true;
         }
      }
      for (int i = 0; i < anz; i++)
      {
         if ((st[i].getfunction() & 1) == 0)
         {
            g.setColor(Color.red);
            g.setXORMode(getBackground());
            g.drawLine(squares[i].x, squares[i].y,
                       maxComponentWidth+squares[i].x, maxComponentHeight+squares[i].y);
            g.drawLine(squares[i].x+1, squares[i].y,
                       maxComponentWidth+squares[i].x+1, maxComponentHeight+squares[i].y);
            g.drawLine(maxComponentWidth+squares[i].x, squares[i].y,
                       squares[i].x, maxComponentHeight+squares[i].y);
            g.drawLine(maxComponentWidth+squares[i].x-1, squares[i].y,
                       squares[i].x-1, maxComponentHeight+squares[i].y);
         }
      }
      if (token_lives()) t.t[0].erase(0, false);
      paintactive = false;
   }

   public boolean addStation()
   {
      if (anz < 15)
      {
         st[anz]= new station();
         st[anz].setfunction(1);
         anz++;
         repaint();
         sta.status.appendText("There are "+anz+" stations in use now.\n");
         return true;
      }
      return false;
   }

   public boolean removeStation()
   {
      boolean b = false;
      if (anz > 2){
//         st[anz-1].setfunction(0);
         anz--;
         b = true;
         sta.status.appendText("There are "+anz+" stations in use now.\n");
      } else b = false;
      repaint();
      return(b);
   }
}

/*****************************************/
/* Stationsklasse für alle Netzstationen */
/*****************************************/
class station
{
   private int funktion = 1; // Stationsbruch:            Bit 2 = 1
                             // Monitor/ normale Station: Bit 1 = 1/0
                             //         oder deaktiviert: Bit 0 = 0
                             //
   private int status = 1; //Sendestatus: idle(1), Sendestation(>100)
   private int dist = 0;

   station()
   {

   }

   public void setdist(int i)
   {
      dist = i;
   }

   public int getdist()
   {
      return dist;
   }

   public void setfunction(int i)
   {
      funktion = i;
   }

   public int getfunction()
   {
      return funktion;
   }

   public void setstatus(int i)
   {
      status = i;
   }

   public int getstatus()
   {
      return status;
   }
}

class SpeedSelect extends Frame {
   private Label currentSpeed;
   private int sleepTime;
   private Scrollbar speed;
   private Button ok,cancel;
   private Font font = new Font("Dialog",Font.BOLD,15);
   private fddi puCl;

   /** constructor Baut neues Fenster auf mit Scrollbar zur Wahl der Geschwindigkeit*/
   public SpeedSelect(fddi publicClass) {
      puCl = publicClass;
      sleepTime = puCl.sleepTime;
      setTitle("Animation speed");
      Panel label = new Panel();
      label.setLayout(new BorderLayout());
      label.add("Center",currentSpeed = new Label("Animation speed (3-500): "+sleepTime+"  "));
      Panel scrollbar = new Panel();
      scrollbar.setLayout(new BorderLayout());
      scrollbar.add("Center",speed = new Scrollbar(Scrollbar.HORIZONTAL,1,1,3,500));
      speed.setPageIncrement(50);
      speed.setLineIncrement(1);
      speed.setValue(sleepTime);
      Panel button = new Panel();
      button.setLayout(new FlowLayout());
      button.add(ok = new Button("Ok"));
      button.add(cancel = new Button ("Cancel"));
      this.setLayout(new BorderLayout());
      this.setFont(font);
      add("North",label);
      add("Center",scrollbar);
      add("South",button);
   }
   public boolean handleEvent(Event evt) {
      if (evt.id == Event.WINDOW_DESTROY) {
         this.hide();
         this.dispose();
      }
      else if (   evt.id ==Event.SCROLL_ABSOLUTE
               || evt.id ==Event.SCROLL_LINE_DOWN
               || evt.id ==Event.SCROLL_LINE_UP
               || evt.id ==Event.SCROLL_PAGE_DOWN
               || evt.id ==Event.SCROLL_PAGE_UP) {
         currentSpeed.setText("Animation speed (3-500): "+speed.getValue()+"  ");
         return true;
      }
      else if(evt.target == ok) {
	 puCl.sleepTime = speed.getValue();
	 if (puCl.netz.token_lives()) puCl.netz.t.sleepTime = speed.getValue();
         this.hide();
         this.dispose();
         return true;
      }
      else if(evt.target == cancel) {
         this.hide();
         this.dispose();
         return true;
      }
      return super.handleEvent(evt);
   }
}

/**************************************************************/
/* Hauptklasse fuer Menueleiste und Buttons                   */
/*                                                            */
/**************************************************************/
public class fddi extends Frame
{
   netpaint netz;
   TextArea status;
   TextField monitorstatus; //tokenstatus;
   Image image1, image2;
   public Button animctrl, runctrl, addButton, removeButton;
   Choice schaden, select;
   ThreadGroup gr;
   int sleepTime = 50;

   public fddi(Image im1, Image im2)
   {
      setTitle("FDDI Simulation V 1.1");
      image1 = im1; image2 = im2;

      //Menu-Bar
      MenuBar mbar = new MenuBar();

      Menu mpr = new Menu("Program");
      mpr.add(new MenuItem("Quit"));
      mbar.add(mpr);

      Menu opt = new Menu("Options");
      opt.add(new MenuItem("Animation speed..."));
      mbar.add(opt);

      setMenuBar(mbar);

      //Selektionsliste
      select = new Choice();
      select.addItem("Send");
      select.addItem("Off/ On");
      select.addItem("Broken ring");

      //Button-Panel
      Panel schalter = new Panel();
      schalter.setLayout(new GridLayout(7,1,20,20));
      schalter.add(new Label("Switches:",Label.CENTER));

      addButton = new Button("Add");
      schalter.add(addButton);

      removeButton = new Button("Remove");
      schalter.add(removeButton);

      animctrl = new Button("Start");
      schalter.add(animctrl);

      runctrl = new Button("Pause");
      schalter.add(runctrl);

      schalter.add(new Label("Selection:",Label.CENTER));
      schalter.add(select);

      //Status-Panel
      Panel statuszeile = new Panel();
      statuszeile.add(new Label("Status:", Label.RIGHT));
      status = new TextArea(5, 60);
      status.setEditable(false);
      status.setBackground(Color.white);
      statuszeile.add(status);

      //Netzfenster erzeugen
      netz = new netpaint(image1, image2, this);

      add("West", schalter);
      add("Center", netz);
      add("South", statuszeile);
   }
   public boolean handleEvent(Event evt)
   {
      if ((evt.id == Event.WINDOW_DESTROY) &&
         (evt.target == this))
         this.dispose();
      return super.handleEvent(evt);
   }

   public boolean action(Event evt, Object arg)
   {
      if (evt.target instanceof MenuItem)
      {
         if (arg.equals("Quit"))
         {
            this.dispose();
         }
         else if (arg.equals("Animation speed..."))
         {
            SpeedSelect spbar = new SpeedSelect(this);
            spbar.pack();
            spbar.show();
         } else return false;
      } else if (evt.target instanceof Choice)
      {
			if (arg.equals("Broken ring"))
         {
            netz.setselectmode(3);
            status.appendText("Select position of ring break.\n");
         } else
			if (arg.equals("Send"))
         {
            netz.setselectmode(1);
				status.appendText("Selection chooses sending or receiving station.\n");
         } else
         if (arg.equals("Off/ On"))
         {
            netz.setselectmode(2);
				status.appendText("Selection switches station on or off.\n");
         } else return false;
      } else
      {
         if (arg.equals("Add"))
         {
            if (!netz.anim_lives())
            {
               if (netz.addStation())
                  status.appendText("Station added.\n");
               else
                  status.appendText("FDDI ring is already full!\n");
            }
         }
         else if (arg.equals("Remove"))
         {
            if (!netz.anim_lives())
	         {
               if (netz.removeStation())
		            status.appendText("Station removed.\n");
	            else
                  status.appendText("Station could not be removed.\n");
	         }
         }
         else if (arg.equals("Start"))
         {
            status.appendText("Token circulates ...\n");
            netz.startanim(sleepTime);
            animctrl.setLabel("Stop");
            addButton.disable();
            removeButton.disable();
         }
         else if (arg.equals("Stop"))
         {
            status.appendText("Token stopped!\n");
            netz.stopanim();
            animctrl.setLabel("Start");
            runctrl.setLabel("Pause"); //zur Sicherheit
            addButton.enable();
            removeButton.enable();
         }
         else if (arg.equals("Pause"))
         {
            if (netz.t != null)
            {
               netz.t.suspend();
               runctrl.setLabel("Continue");
            }
         }
         else if (arg.equals("Continue"))
         {
            netz.t.resume();
            runctrl.setLabel("Pause");
         }
         else return false;
      }
      return true;
   }
}
